/**
 * Copyright (c) 2007 Colony Project
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 **/

/**
 * @file sensor_matrix.h
 * @brief Definitions for sensor matrices
 *
 * Contains functions and declarations for using sensor matrices.
 *
 * @author Brian Coltin, Colony Project, CMU Robotics Club
 **/

#ifndef SENSOR_MATRIX_H
#define SENSOR_MATRIX_H


/**
 * @defgroup sensormatrix Sensor Matrix
 * @brief the robot sensor matrix
 *
 * These functions and structures are used for localization
 * to determine the relative directions of robots.
 *
 * @{
 **/

#define MAXIMUM_XBEE_ID		0x10
#define READING_UNKNOWN		0xFF

/**
 * @struct SensorMatrix
 *
 * A sensor matrix.
 **/
//TODO: the order of member variables in this struct should be changed in case the compile packs the struct
// In order to achieve the best packing, the variables should be listed in order of decreasing memory size.
// Thus, pointers should be first, followed by int, followed by char.
typedef struct
{
	/**
	 * The number of robots in the token ring.
	 **/
	int numJoined;
	/**
	 * The element representing a robot is true if that robot
	 * is in the token ring and false otherwise.
	 **/
	unsigned char joined[MAXIMUM_XBEE_ID];

	// on the bayboard, we don't include the matrix to save memory.
#ifndef BAYBOARD
	/**
	 * The matrix. Each row represents the readings of one
	 * robot.
	 **/
	unsigned char matrix[MAXIMUM_XBEE_ID][MAXIMUM_XBEE_ID];
#endif
} SensorMatrix;

/**@brief Create a sensor matrix **/
void sensor_matrix_create(void);
/**@brief Set a reading in a sensor matrix **/
void sensor_matrix_set_reading(int observer, int robot, int reading);
/**@brief Get a reading in a sensor matrix **/
int sensor_matrix_get_reading(int observer, int robot);
/**@brief Set whether the robot is in the token ring **/
void sensor_matrix_set_in_ring(int robot, int in);
/**@brief Get whether the robot is in the sensor ring **/
int sensor_matrix_get_in_ring(int robot);
/**@brief Get the number of robots which have joined the token ring **/
int sensor_matrix_get_joined(void);
/**@brief Get the maximum size of the sensor matrix **/
int sensor_matrix_get_size(void);

/** @} **/ //end defgroup


#endif

