/**
 * Copyright (c) 2007 Colony Project
 * 
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 **/

/**
 * @file wl_token_ring.h
 * @brief Declarations for the token ring packet group
 * 
 * Contains declarations for the token ring packet group.
 *
 * @author Brian Coltin, Colony Project, CMU Robotics Club
 **/

#ifndef WL_TOKEN_RING_H
#define WL_TOKEN_RING_H

/**
 * @defgroup tokenring Token Ring
 * @brief Wireless library token ring implementation
 *
 * This packet group is used to form a token ring, which
 * keeps track of the relative directions of the robots
 * from one another.
 *
 * @{
 **/

/**@brief Register the token ring group with the wireless library.**/
int wl_token_ring_register(void);
/**@brief Unregister the token ring group with the wirelss library.**/
void wl_token_ring_unregister(void);
/**@brief Set the functions called to turn the bom on and off.**/
void wl_token_ring_set_bom_functions(void (*on_function) (void), void (*off_function) (void),
  int (*max_bom_function) (void));

/**@brief Join the token ring **/
int wl_token_ring_join(void);
/**@brief Leave the token ring **/
void wl_token_ring_leave(void);

/**@brief Return the number of robots in the token ring **/
int wl_token_get_robots_in_ring(void);
/**@brief Return whether a given robot is in the token ring **/
int wl_token_is_robot_in_ring(int robot);

/**@brief Begin iterating through robots in the token ring **/
void wl_token_iterator_begin(void);
/**@brief Returns whether there are more robots to iterate through **/
int wl_token_iterator_has_next(void);
/**@brief Returns the ID of the next robot in the token ring **/
int wl_token_iterator_next(void);

/**@brief Return the latest BOM reading between two robots **/
int wl_token_get_sensor_reading(int source, int dest);
/**@brief Return the latest BOM reading between us and another robot **/
int wl_token_get_my_sensor_reading(int dest);
/**@brief Return the number of robots in the sensor matrix.*/
int wl_token_get_num_robots(void);
/**@brief Return the number of non-null elements in the sensor matrix*/
int wl_token_get_matrix_size(void);

/** @} **/ //end token ring group

#endif
