/**
 * Copyright (c) 2007 Colony Project
 * 
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 **/

/**
 * @file xbee.h
 * @brief Contains definitions for using the XBee
 *
 * Contains definitions for interfacing with the 
 * XBee module, from either a robot or a computer.
 * To use a robot, define ROBOT in wl_defs.h, and
 * to use a computer, don't define ROBOT.
 *
 * @author Brian Coltin, Colony Project, CMU Robotics Club
 **/

#ifndef XBEE_H
#define XBEE_H

/**
 * The port to use the XBee from on the computer.
 * Also, a backup port if the other is used.
 **/
#ifndef ROBOT
#define XBEE_PORT_DEFAULT "/dev/ttyUSB1"
#endif

/**
 * @defgroup xbee XBee
 * @brief Interface with the XBee module
 *
 * Interface with the XBee module.
 *
 * @{
 **/

/*Definitions*/
/**@brief Unset PAN, uses XBee default **/
#define XBEE_PAN_DEFAULT 0xFFFF
/**@brief Unset channel, uses XBee default **/
#define XBEE_CHANNEL_DEFAULT 0
/**@brief Broadcast to all robots in the PAN **/
#define XBEE_BROADCAST 0xFFFF
/**@brief No special options **/
#define XBEE_OPTIONS_NONE 0x00
/**@brief Do not receive a TX_STATUS message from this packet **/
#define XBEE_OPTIONS_DISABLE_RESPONSE 0x01
/**@brief Send the packet to all PANS **/
#define XBEE_OPTIONS_BROADCAST_ALL_PANS 0x04
/**@brief A transmit status packet **/
#define XBEE_TX_STATUS 0x89
/**@brief A packet received from another XBee **/
#define XBEE_RX 0x81

/**@brief Initialize the XBee library **/
int xbee_lib_init(void);
/**@brief Uninitialize the XBee library **/
void xbee_terminate(void);
/**@brief Get a packet from the XBee **/
int xbee_get_packet(unsigned char* packet);
/**@brief Send a packet to the XBee **/
int xbee_send_packet(char* packet, int len, int dest, char options, char frame);
/**@brief Set the PAN ID for the XBee **/
int xbee_set_pan_id(int id);
/**@brief Get the XBee's PAN ID **/
unsigned int xbee_get_pan_id(void);
/**@brief Set the channel the XBee is currently using **/
int xbee_set_channel(int channel);
/**@brief Get the channel the XBee is currently using **/
int xbee_get_channel(void);
/**@brief Get the XBee's 16-bit address **/
unsigned int xbee_get_address(void);
/**@brief Set the com port on a computer, undefined on the robot**/
void xbee_set_com_port(char* port);

/**@}**/ //end defgroup

#endif
